# from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse

import floppyforms as forms
from crispy_forms.layout import Submit, Button
from crispy_forms.helper import FormHelper
from crispy_forms.bootstrap import FormActions

from .models import Attachment


class AttachmentForm(forms.ModelForm):

    next = forms.CharField(widget=forms.HiddenInput())

    class Meta:
        model = Attachment
        fields = ('attachment_file', 'filetype', 'author', 'title', 'legend')

    def __init__(self, request, *args, **kwargs):
        self._object = kwargs.pop('object', None)
        next_url = kwargs.pop('next_url', None)

        super(AttachmentForm, self).__init__(*args, **kwargs)
        self.fields['legend'].widget.attrs['placeholder'] = _('Sunset on lake')
        self.fields['author'].initial = request.user

        # Detect fields errors without uploading (using HTML5)
        self.fields['filetype'].widget.attrs['required'] = 'required'
        self.fields['author'].widget.attrs['pattern'] = '^[A-Za-z0-9]+'
        self.fields['legend'].widget.attrs['pattern'] = '^[A-Za-z0-9]+'

        next_url = request.POST.get('next') or next_url
        next_url = next_url or request.GET.get('next', '/')
        self.fields['next'].initial = next_url

        self.helper = FormHelper(form=self)
        self.helper.form_tag = True
        self.helper.form_class = 'attachment form-horizontal'
        self.helper.help_text_inline = True

        is_creation = not self.instance.pk

        if is_creation:
            # Mark file field as mandatory
            file_field = self.fields['attachment_file']
            file_field.widget.attrs['required'] = 'required'

            form_url = reverse('add_attachment', kwargs={
                'app_label': self._object._meta.app_label,
                'module_name': self._object._meta.module_name,
                'pk': self._object.pk
            })
            form_actions = [
                Submit('submit_attachment',
                       _('Submit attachment'),
                       css_class="btn-primary offset1")
            ]

        else:
            # When editing an attachment, changing its title won't rename!
            self.fields['title'].widget.attrs['readonly'] = True
            form_url = reverse('update_attachment', kwargs={
                'attachment_pk': self.instance.pk
            })
            form_actions = [
                Button('cancel', _('Cancel'), css_class=""),
                Submit('submit_attachment',
                       _('Update attachment'),
                       css_class="btn-primary offset1")
            ]

        self.helper.form_action = form_url
        self.helper.layout.fields.append(
            FormActions(*form_actions, css_class="form-actions"))

    def success_url(self):
        return self.cleaned_data.get('next')

    def save(self, request, *args, **kwargs):
        obj = self._object
        self.instance.creator = request.user
        self.instance.content_type = ContentType.objects.get_for_model(obj)
        self.instance.object_id = obj.id
        return super(AttachmentForm, self).save(*args, **kwargs)



from django.conf import settings


app_settings = dict({
    'FILETYPE_MODEL': 'FileType',
    'ATTACHMENT_TABLE_NAME': 'paperclip_attachment',
    'ACTION_HISTORY_ENABLED': True,
}, **getattr(settings, 'PAPERCLIP_CONFIG', {}))



import os
import mimetypes

from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from paperclip import app_settings


FILETYPE_MODEL = app_settings['FILETYPE_MODEL']


class FileType(models.Model):
    type = models.CharField(max_length=128, verbose_name=_("File type"))

    class Meta:
        abstract = FILETYPE_MODEL != 'FileType'
        verbose_name = _(u"File type")
        verbose_name_plural = _(u"File types")
        ordering = ['type']

    def __unicode__(self):
        return self.type


class AttachmentManager(models.Manager):

    def attachments_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        return self.filter(content_type__pk=object_type.id,
                           object_id=obj.id)


def attachment_upload(instance, filename):
    """Stores the attachment in a "per module/appname/primary key" folder"""
    name, ext = os.path.splitext(filename)
    renamed = slugify(instance.title or name) + ext
    return 'paperclip/%s/%s/%s' % (
        '%s_%s' % (instance.content_object._meta.app_label,
                   instance.content_object._meta.object_name.lower()),
        instance.content_object.pk,
        renamed)


class Attachment(models.Model):

    objects = AttachmentManager()

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    attachment_file = models.FileField(_('Attachment'),
                                       upload_to=attachment_upload,
                                       max_length=512)
    filetype = models.ForeignKey(FILETYPE_MODEL, verbose_name=_('File type'))

    creator = models.ForeignKey(User, related_name="created_attachments",
                                verbose_name=_('Creator'),
                                help_text=_("User that uploaded"))
    author = models.CharField(blank=True, default='', max_length=128,
                              db_column='auteur', verbose_name=_('Author'),
                              help_text=_("Original creator"))
    title = models.CharField(blank=True, default='', max_length=128,
                             db_column='titre', verbose_name=_(u"Filename"),
                             help_text=_("Renames the file"))
    legend = models.CharField(blank=True, default='', max_length=128,
                              db_column='legende', verbose_name=_(u"Legend"),
                              help_text=_("Details displayed"))
    starred = models.BooleanField(default=False, db_column='marque',
                                  verbose_name=_(u"Starred"),
                                  help_text=_("Mark as starred"))

    date_insert = models.DateTimeField(editable=False, auto_now_add=True,
                                       verbose_name=_(u"Insertion date"))
    date_update = models.DateTimeField(editable=False, auto_now=True,
                                       verbose_name=_(u"Update date"))

    class Meta:
        db_table = app_settings['ATTACHMENT_TABLE_NAME']
        ordering = ['-date_insert']
        verbose_name = _(u"Attachment")
        verbose_name_plural = _(u"Attachments")
        permissions = (
            ('read_attachment', _('Can read attachments')),
            ('delete_attachment_others', _("Can delete others' attachments")),
        )

    def __unicode__(self):
        return '%s attached %s' % (self.creator.username,
                                   self.attachment_file.name)

    @property
    def filename(self):
        return os.path.split(self.attachment_file.name)[1]

    @property
    def mimetype(self):
        mt = mimetypes.guess_type(self.attachment_file.name, strict=True)[0]
        if mt is None:
            return ('application', 'octet-stream')
        return mt.split('/')

    @property
    def is_image(self):
        return self.mimetype[0].startswith('image')
